#!/usr/bin/python3.6
'''
Created on 24 Jul 2018

@author: mohammed
'''

def register(target, subject,points=20):

    r""" Make StreamlineLinearRegistration simpler to use

    Parameters:
    ----------
    :param target: List of numpy.ndarray,
        it is the target bundle witch will be static during registration
    :param subject:List of numpy.ndarray,
        it is the target bundle witch will be moving during registration
    :param points: int,
        The bundles will be divided to this number
    :return: List of numpy.ndarray,
        It the aligned subject to target,
        The function will print out the transformation matrix as well.
    """
    from dipy.align.streamlinear import StreamlineLinearRegistration
    from dipy.tracking.streamline import set_number_of_points

    cb_subj1 = set_number_of_points(target, points)
    cb_subj2 = set_number_of_points(subject, points)

    srr = StreamlineLinearRegistration()
    srm = srr.optimize(static=cb_subj1, moving=cb_subj2)
    del cb_subj1
    del cb_subj2
    del target
    return srm.transform(subject),srm.matrix

def register_all(data_path):

    r""" Register all ply files in a folder

    :param data_path: str,
        - A folder has subjects each in a folder as ply file format.
        - It will not read ply images putted directly in this folder but inside folders.
        - The subject in the first folder will be targets and the others are moved subjects
    :return: files,
        It wil export aligned subject to trk files each in a new folder as the same name as subject plus _out
    """
    import time
    from os import listdir, mkdir
    from os.path import isfile, isdir
    from tractography.io import read_ply,write_trk,write_ply

    time_list={}
    dirs=[dir for dir in listdir(data_path)]
    target_dir=data_path+'/'+dirs[0]
    files = [f for f in listdir(target_dir) if isfile(target_dir+'/'+f) and f.endswith('.ply')]
    for f in files:
        time_list[f]={}
        start_time = time.clock()
        target = read_ply(target_dir+'/'+f)
        time_list[f]['Loading Target']=time.clock()-start_time
        for i in range(1,len(dirs)):
            subject_path = data_path+'/'+dirs[i]+'/'+f
            out_path = data_path+'/'+dirs[i]+'/out_'+f
            if isfile(subject_path):
                start_time = time.clock()
                subject=read_ply(subject_path)
                time_list[f]['Loading Subject '+dirs[i]]=time.clock()-start_time
                start_time = time.clock()
                aligned_subject = register(target=target,subject=subject)
                time_list[f]['Align Subject '+dirs[i]]=time.clock()-start_time
                start_time = time.clock()
                write_ply(out_path, aligned_subject)
                write_trk(out_path+'.trk', aligned_subject)
                time_list[f]['Writing '+dirs[i]]=time.clock()-start_time
    del dirs
    del target_dir
    del files
    del out_path
    del start_time
    del subject_path
    del target
    del subject
    del aligned_subject
    return time_list


#!/usr/bin/python3.6
'''
Created on 24 Jul 2018

@author: mohammed
'''
def read_ply(fname):
    """
    Read ply file and return bundles data
    :param fname:
    :return:
    """
    from plyfile import PlyData, PlyElement
    import numpy as np

    ply = PlyData.read(fname)
    data = []
    indeces = np.asarray([i[0] for i in ply.elements[1].data])
    temp = []
    for i, item in enumerate(ply.elements[0].data):
        i += 1
        temp.append([item[3], item[4], item[5]])
        if i in indeces:
            data.append(np.asarray(temp))
            temp = []
    del temp
    del indeces
    del ply
    return data

def write_ply(fname, data,
              comments=['DTI Tractography, produced by fiber-track']):
    """
    Write bundles to ply file
    :param fname:
    :param data:
    :param comments:
    :return:
    """
    data_txt = ''
    vert_num = 0
    indeces = []
    index = 0
    with open(fname, 'w') as f:
        for fib in data:
            index += len(fib)
            indeces.append(index)
            for vert in fib:
                data_txt += str(vert[0]) + ' ' + str(vert[1]) + ' ' + str(vert[2]) + '\n'
                vert_num += 1
        f.write('ply\nformat ascii 1.0\n')
        for com in comments:
            f.write('comment ' + com + '\n')
        f.write('element vertices ' + str(vert_num))
        f.write('\nproperty float x\nproperty float y\nproperty float z\n')
        f.write('element fiber {}\nproperty int endindex'.format(len(indeces)))
        f.write('\nend_header\n' + data_txt)
        for i in indeces:
            f.write(str(i) + '\n')

    del data_txt
    del indeces

def write_trk(fname,data):

    r""" Write bundles to trk file format

    :param fname: str,
        It is the output file name
    :param data: List of numpy.ndarray,
        The bundle to be written
    :return: trk file,
        Export images as trk file format
    """
    import numpy as np
    from dipy.io.streamline import save_trk
    save_trk(fname, streamlines=data, affine=np.eye(4))

def export_bundles(bundles, colors=None, show=True, fname=None):

    r""" Visualize and export bundles

    :param bundles: List of numpy.ndarray,
        Bundles to be exported
    :param colors: window.colors
    :param show: Boolean,
        True to show the image(s) or False
    :param fname: str,
        Output file name
    :return:
        Images file(s) and/or show them
    """
    from dipy.viz import window, actor
    from time import sleep

    ren = window.Renderer()
    ren.SetBackground(1, 1, 1)
    for (i, bundle) in enumerate(bundles):
        color = colors[i]
        lines_actor = actor.streamtube(bundle, color, linewidth=0.3)
        lines_actor.RotateX(-90)
        lines_actor.RotateZ(90)
        ren.add(lines_actor)
    if show:
        window.show(ren)
    if fname is not None:
        sleep(1)
        window.record(ren, n_frames=1, out_path=fname, size=(900, 900))



#!/usr/bin/python3.6
from setuptools import setup, find_packages
#from shutil import copyfile
#from os import remove
#from time import sleep
#from os.path import  isfile
"""
if isfile("../README.md"):
    copyfile("../README.md","README.md")
"""

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="tractography",
    version="0.1.12",
    author="Mohammed Abdelgadir",
    description="The package deals with brain bundles images",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://weekmo.github.io/Tractography/",
    packages=find_packages(exclude=["*.test", "*.test.*", "test.*", "test"]),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ),
)
"""
sleep(2)
remove("README.md")
"""